// This fragment shader can:
// - displace pixels from ("cam") based on an input video ("disp")
// - ASCIIify the video, mapping grayscale intensity to the 10x10 ASCII tileset ("font") based on the grid size ("gridSize").
//
precision mediump float;

uniform sampler2D cam;
uniform sampler2D font;
uniform sampler2D disp;

uniform float canvasRatio;
uniform float dispScale;
uniform float charOffsetScale;
uniform float gridSize;
uniform float glyphs[10];
uniform float greyScale;
uniform float time;

uniform float displacementYMapOffset;
uniform float displacementYMapScale;

varying vec2 vTexCoord;

void main(){
  // Initialize ASCII tileset
  float fontTexGridsize = 1.0/10.; // font Tex is a 10x10 grid so divide the canvas (1.0) by 10
  vec2 cell_count = vec2(gridSize * canvasRatio, gridSize ); // Adjust the horizontal gridsize dynamically to "add more columns" as needed and avoid text scaling
  
  // "pixelate" the UV value and grab the displacement from the underlying video
  vec2 pixelUV = floor(vTexCoord*cell_count)/cell_count;
  vec2 displacementMapUV = vec2(pixelUV[0], displacementYMapOffset + displacementYMapScale*pixelUV[1]);
  vec4 dispTex = texture2D(disp, displacementMapUV);
  dispTex.rgb = dispTex.rgb * 2.0 - 1.0;

  vec2 displacement = dispTex.rb * dispScale;
  displacement = floor(displacement*cell_count)/cell_count;
    
  // Displace UV based on the underlying video
  vec2 uv = pixelUV + displacement;
  uv.y = 1.0 - uv.y; // flip UV so it's right-way-up

  // Calculate character offsets to embed an animated pattern
  float t = time * 0.1;
	float x = abs(floor(120.*pixelUV.x) - 60.);
	float y = abs(floor(50.*pixelUV.y) - 40.);
	float o = sin(y * x * cos((x + y) * 0.001 * t) * 0.003 + y * 0.01 + t) * 40.;

  float offset = charOffsetScale*(floor(x + y + o + 20.*displacement[0]));
  
  // Repeat the single glyph tex and generate tiled patterns for each glyph
  // Make sure to use the unpixelated UV so the glyphs don't get distorted.
  vec2 unpixelatedUV = vTexCoord + displacement;
  unpixelatedUV.y = 1.0 - unpixelatedUV.y;
  
  vec2 glyphTexUv = mod(unpixelatedUV * cell_count, 1.);

  vec2 glyph1 = vec2(mod(glyphs[0] + offset, 9.), 0.);  
  vec2 glyph2 = vec2(mod(glyphs[1] + offset, 9.), 0.);
  vec2 glyph3 = vec2(mod(glyphs[2] + offset, 9.), 0.);
  vec2 glyph4 = vec2(mod(glyphs[3] + offset, 9.), 0.);
  vec2 glyph5 = vec2(mod(glyphs[4] + offset, 9.), 0.);
  vec2 glyph6 = vec2(mod(glyphs[5] + offset, 9.), 0.);
  vec2 glyph7 = vec2(mod(glyphs[6] + offset, 9.), 0.);
  vec2 glyph8 = vec2(mod(glyphs[7] + offset, 9.), 0.);
  vec2 glyph9 = vec2(mod(glyphs[0], 9.), 0.); // BACKGROUND
  vec2 glyph10 = vec2(mod(glyphs[9] + offset, 10.), 0.);
  vec2 glyph11 = vec2(mod(glyphs[9] + offset, 10.), 0.);

  vec2 glyphUv1 = (glyphTexUv+glyph1) * fontTexGridsize; // glyph selection
  vec2 glyphUv2 = (glyphTexUv+glyph2) * fontTexGridsize; 
  vec2 glyphUv3 = (glyphTexUv+glyph3) * fontTexGridsize; 
  vec2 glyphUv4 = (glyphTexUv+glyph4) * fontTexGridsize; 
  vec2 glyphUv5 = (glyphTexUv+glyph5) * fontTexGridsize; 
  vec2 glyphUv6 = (glyphTexUv+glyph6) * fontTexGridsize; 
  vec2 glyphUv7 = (glyphTexUv+glyph7) * fontTexGridsize; 
  vec2 glyphUv8 = (glyphTexUv+glyph8) * fontTexGridsize; 
  vec2 glyphUv9 = (glyphTexUv+glyph9) * fontTexGridsize; 
  vec2 glyphUv10 = (glyphTexUv+glyph10) * fontTexGridsize; 
  vec2 glyphUv11 = (glyphTexUv+glyph11) * fontTexGridsize; 

  vec4 tiledGlyphTex1 = 1. - texture2D(font, glyphUv1); // applying UV to texture
  vec4 tiledGlyphTex2 = 1. - texture2D(font, glyphUv2); 
  vec4 tiledGlyphTex3 = 1. - texture2D(font, glyphUv3); 
  vec4 tiledGlyphTex4 = 1. - texture2D(font, glyphUv4);
  vec4 tiledGlyphTex5 = 1. - texture2D(font, glyphUv5);
  vec4 tiledGlyphTex6 = 1. - texture2D(font, glyphUv6); 
  vec4 tiledGlyphTex7 = 1. - texture2D(font, glyphUv7); 
  vec4 tiledGlyphTex8 = 1. - texture2D(font, glyphUv8); 
  vec4 tiledGlyphTex9 = 1. - texture2D(font, glyphUv9); 
  vec4 tiledGlyphTex10 = 1. - texture2D(font, glyphUv10); 
  vec4 tiledGlyphTex11 = 1. - texture2D(font, glyphUv11); 
  
  vec4 tex = texture2D(cam,uv);
  float avgTex = max(max(tex.r, tex.g), tex.b); // make grayscale

  // Generate ASCII masks based on grayscale pixel color
  float mask1 = 1.0 - step(0.05,avgTex); // masking between 2 values — 1 (if val < 0.05) or 0 (if 0.05 < val)
  float mask2 = step(0.05,avgTex) - step(0.15,avgTex);
  float mask3 = step(0.15,avgTex) - step(0.25,avgTex);
  float mask4 = step(0.25,avgTex) - step(0.35,avgTex);
  float mask5 = step(0.35,avgTex) - step(0.45,avgTex);
  float mask6 = step(0.45,avgTex) - step(0.55,avgTex);
  float mask7 = step(0.55,avgTex) - step(0.65,avgTex);
  float mask8 = step(0.65,avgTex) - step(0.75,avgTex);
  float mask9 = step(0.75,avgTex) - step(0.85,avgTex);
  float mask10 = step(0.85,avgTex) - step(0.95,avgTex);
  float mask11 = step(0.95,avgTex) - step(1.1,avgTex);

  // Only one of these glyph layers is expected based on the masks above, which are mutually exclusive
  float glyphLayer1 = mask1;
  float glyphLayer2 = min(1.0, (tiledGlyphTex10.r) + (1.0- mask2));
  float glyphLayer3 = min(1.0, (tiledGlyphTex9.r) + (1.0- mask3));
  float glyphLayer4 = min(1.0, (tiledGlyphTex8.r) + (1.0- mask4));
  float glyphLayer5 = min(1.0, (tiledGlyphTex7.r) + (1.0- mask5));
  float glyphLayer6 = min(1.0, (tiledGlyphTex6.r) + (1.0- mask6));
  float glyphLayer7 = min(1.0, (tiledGlyphTex5.r) + (1.0- mask7));
  float glyphLayer8 = min(1.0, (tiledGlyphTex5.r) + (1.0- mask8));
  float glyphLayer9 = min(1.0, (tiledGlyphTex4.r) + (1.0- mask9));
  float glyphLayer10 = min(1.0, (tiledGlyphTex4.r) + (1.0- mask10));
  float glyphLayer11 = min(1.0, (tiledGlyphTex1.r) + (1.0- mask11));

  // Apply all the masked glyph layers, which should just be one of the glyph layer tiles.
  float greyscale = (glyphLayer2 * glyphLayer3 * glyphLayer4 *glyphLayer5*glyphLayer6*glyphLayer7*glyphLayer8*glyphLayer9*glyphLayer10*glyphLayer11);
  vec4 greyScaleTex = vec4(vec3(greyscale),1.0);
  gl_FragColor = tex - greyScaleTex;
}